/******************************************************************************
 * matrix.c
 * Federico Bareilles
 * 08/02/2003
 *
 *
 *****************************************************************************/
#define VERSION "0.1-0"

#define _POSIX_SOURCE 1         /* POSIX compliant source  */

#include <stdio.h>
#include <stdlib.h>
#include "matrix.h"

/******************************************************************************
 * Internal functionos:
 */


int sort_0_row(struct matrix *A, int *ind)
{
  int i, j, c;
  int count[A->d[0]];

  for(i=0;i<A->d[0];i++){
    count[ind[i]] = 0;
    for(j=0;j<A->d[1];j++){
      if( A->M[ind[i]][j] == 0.0 )count[ind[i]]++;
    }
  }
  for(i=0;i<A->d[0];i++)
    for(j=i;j<A->d[0];j++)
      if(count[ind[i]] < count[ind[j]]){
	c = ind[i];
	ind[i] = ind[j];
	ind[j] = c;
      }

  return 0;
}


int swap_row(struct matrix *A, int ia, int ib)
{
  int j;
  double dumi;

  if(ia == ib) return 1;
  for(j=0;j<A->d[1];j++){
    dumi = A->M[ia][j];
    A->M[ia][j] = A->M[ib][j];
    A->M[ib][j] = dumi;
  }

  return -1;
}


long double det(struct matrix *M)
{
  int i, signo;
  long double d = 0.0;
  struct matrix *A;

  /* print_matrix(M); */
  if(M->d[0] == 2 || M->d[0] == 1 ){
    if(M->d[0] == 2)
      d = (long double) (M->M[0][0] * M->M[1][1] - M->M[1][0] * M->M[0][1]);
    else
      d = (long double) M->M[0][0];
  }
  else{
    signo = 1;
    for(i=0;i<M->d[1];i++){
      if(M->M[0][i] != 0.0){
	A = sub_matrix(M,0,i);
	d += (long double) M->M[0][i] * det(A) * ( long double) signo;
	free_matrix(A);
      }
      signo *= -1;
    }
  }

  return d;
}


/******************************************************************************
 * Public functionos:
 */


struct matrix *make_matrix(int n, int m)
{
  int i;
  struct matrix *M;
  
  M = (struct matrix *) malloc(sizeof(struct matrix));
  M->d[0] = n;
  M->d[1] = m;

  M->M = (double **) malloc(sizeof(double) * n);
  for(i=0;i<n;i++)
    M->M[i] = (double *) malloc(sizeof(double) * m);

  return M;
}


int print_matrix(struct matrix *M)
{
  int i,j;

  if(M != NULL){
    for(i=0;i<M->d[0];i++){
      printf("| ");
      for(j=0;j<M->d[1];j++){
	printf("%12.6f ",M->M[i][j]);
      }
      printf("|\n");
    }
    printf("\n");
  }
  return 0;
}

    
int free_matrix(struct matrix *M)
{
  int i;
  if( M != NULL ){
    for(i=0;i<M->d[0];i++)
      free(*(M->M+i));
    free(M->M);
    free(M);
  }
  return 0;
}


struct matrix *copy_matrix(struct matrix *A)
{
  int i, j;
  struct matrix *M;

  M = make_matrix(A->d[0],A->d[1]);
  for(i=0;i<M->d[0];i++){
    for(j=0;j<M->d[1];j++){
      M->M[i][j] = A->M[i][j];
    }
  }

  return M;
}


struct matrix *prod_matrix(struct matrix *A, struct matrix *B)
{
  int i, j, k;
  struct matrix *M;

  if(A->d[1] != B->d[0]) return NULL;

  M = make_matrix(A->d[0],B->d[1]);
  for(i=0;i<M->d[0];i++){
    for(j=0;j<M->d[1];j++){
      M->M[i][j] = 0.0;
      for(k=0; k < A->d[1]; k++){
        M->M[i][j] += A->M[i][k] * B->M[k][j];
      }
    }
  }

  return M;
}


struct matrix *trans_matrix(struct matrix *A)
{
  int i, j;
  struct matrix *M;

  M = make_matrix(A->d[1],A->d[0]);
  for(i=0;i<M->d[0];i++){
    for(j=0;j<M->d[1];j++){
      M->M[i][j] = A->M[j][i];
    }
  }

  return M;
}


struct matrix *sub_matrix(struct matrix *A, int ci, int cj)
{
  int mi, mj, i, j;
  struct matrix *M;

  mi = mj = 0;
  M = make_matrix(A->d[1]-1,A->d[0]-1);

  for(i=0; i < A->d[0]; i++){
    if(i != ci){
      for(j=0; j < A->d[1]; j++){
	if(j != cj){
	  M->M[mi][mj] = A->M[i][j];
	  mj++;
	}
      }
      mi++;
      mj = 0;
    }
  }

  return M;
}


double det_matrix(struct matrix *M)
{
  int i, j;
  int s = 1;
  int ind[M->d[0]];
  struct matrix *A;
  double d;

  A = copy_matrix(M);                   /* Make e copy */
  for(i=0;i<A->d[0];i++)ind[i] = i;     
  sort_0_row(A,ind);                    /* sort */
  for(i=0;i<A->d[0];i++){               /* Sort de copy which 0 first */
    if(ind[i] != i){
      s *= swap_row(A,i,ind[i]);
      j = 0;
      while(ind[j] != i)j++;
      ind[j] = ind[i];
      ind[i] = i;
    }
  }
  d = ( double )det(A)*s;               /* Reali call determinat runtime */
  free_matrix(A);                       /* free copy of matrix */

  return d;
}


struct matrix *adj_matrix(struct matrix *M)
{
  int i, j;
  int sig_r = 1;
  int sig_c = 1;
  struct matrix *A,*S;
  
  A = make_matrix(M->d[0],M->d[1]);
  if( M->d[0] == 1 ) A->M[0][0] = 1.;
  else{
    for(i=0;i<M->d[0];i++){
      for(j=0;j<M->d[1];j++){
	S = sub_matrix(M,j,i);
	A->M[i][j] =  det_matrix(S) * (double) (sig_r * sig_c);
	sig_c *= -1;
	free_matrix(S);
      }
      sig_r *= -1;
      sig_c = 1;
    }
  }
  
  return A;
}


struct matrix *inv_matrix(struct matrix *M)
{
  int i, j;
  int sig_r = 1;
  int sig_c = 1;
  struct matrix *A,*S;
  double d;

  A = NULL;
  d = det_matrix(M);
  if( d != 0.0 ){
    A = make_matrix(M->d[0],M->d[1]);
    if( M->d[0] == 1 ) A->M[0][0] = 1./d;
    else{
      for(i=0;i<M->d[0];i++){
	for(j=0;j<M->d[1];j++){
	  S = sub_matrix(M,j,i);
	  A->M[i][j] =  det_matrix(S) * (double) (sig_r * sig_c) / d;
	  sig_c *= -1;
	  free_matrix(S);
	}
	sig_r *= -1;
	sig_c = 1;
      }
    }
  }

  return A;
}


/*
 * End matrix.
 *****************************************************************************/
