/******************************************************************************
 * no_linear_fit.c
 * Federico Bareilles
 * 10/02/2003
 *
 *
 *****************************************************************************/

#define _POSIX_SOURCE 1         /* POSIX compliant source  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "tipper_red.h"
#include "matrix.h"
#include "minimos.h"


double mi_modelo(double x, double *xx)
{
  double c = *(xx+0);
  double t = *(xx+1);
  double d = *(xx+2);

  return d + c * exp(-t * x);
}

double mi_deriv_c(double x, double *xx)
{
  //double c = *(xx+0);
  double t = *(xx+1);
  //double d = *(xx+2);

  return exp(-t * x);
}

double mi_deriv_t(double x, double *xx)
{
  double c = *(xx+0);
  double t = *(xx+1);
  //double d = *(xx+2);

  return -1.0 * x * c *exp(-t * x);
}

double mi_deriv_d(double x, double *xx)
{
  //double c = *(xx+0);
  //double t = *(xx+1);
  //double d = *(xx+2);
  //double a = *(xx+3);

  return 1.0;
}

/****************************************************************************/


struct fit *no_linear_fit(float *xd, float *yd, float *sd, 
			  int num, float Ta, float Tl)
{
  int i;
  struct fit *f;
  double x[4];
  double xdata[num], ydata[num], sigma[num];
  struct model *m;

  f = (struct fit *) malloc(sizeof(struct fit));

  for(i=0; i<num;i++){
    xdata[i] = (double) xd[i];
    ydata[i] = (double) yd[i];
    sigma[i] = (double) sd[i];
    //printf("#%f %f %f\n",xdata[i], ydata[i],sigma[i]); 
  }

  m = make_model(mi_modelo);
  model_add_deriv(m, mi_deriv_c);
  model_add_deriv(m, mi_deriv_t);
  model_add_deriv(m, mi_deriv_d);
  
  m->xdata = xdata;
  m->ydata = ydata;
  m->sigma = sigma;
  m->size = num;

  Ta = Ta - 10.3092; /* calibrado con la campanna del burek */
  //Tl = Tl - 3.5852;
  x[0] = .52 * Ta;   //c
  x[1] = .3;         //t 
  x[2] = .52 * (Tl-Ta);// *  d[0];    //d 

  /*
  fit_model(m,x);
  x[2] = x[0] / Ta *(Tl-Ta);
  //tmp = x[1];
  */

  if( fit_model(m,x) == 0 ){
    
    printf("# Tau: %f, gnTa: %f, gn: %f, Dt: %f\n",
	   x[1],x[0],x[0]/Ta, Tl-Ta);
    
    f->tau = x[1];
    f->sigma_tau = sqrt(m->VCV->M[1][1]);
    f->b = f->sigma_b = f->resid = f->chi2f = 0.0;
    f->b=x[0]/Ta;
    for(i=0;i<num;i++){
      f->resid += SQR(m->ydata[i] - mi_modelo(m->xdata[i], x) );
      f->chi2f += SQR((m->ydata[i] - mi_modelo(m->xdata[i], x))/m->sigma[i]);
      //printf("# %f %f %f %f\n",xdata[i], ydata[i],m->sigma[i],mi_modelo(m->xdata[i], x) );      
    }
    f->resid = sqrt(f->resid);
    return f;
  }else{
    free(f);
    return NULL;
  }

}
