/******************************************************************************
 * zener_table.c
 * Federico Bareilles
 * 31/01/2003
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "tipper_red.h"


struct zener{
  float in;
  float out;
  struct zener *next;
};


/*** CHANGE HISTORY:
        Nov 95  DMcG    Vin and Vout have been rediefined:
                           Vin  = is the voltage at the input of the A/D
                                 after the zener diode.
                           Vout = is the voltage at the input of the zener
                                  diode circuit.
                        The zener interpolation correction must be performed
                        AFTER the reported voltage has been corrected for
                        ref voltage calibration error.
***/

//struct zener_table *get_table_zener(char *file)
struct zener_table *get_table_zener(struct instrument *t)
{
  int count = 0;
  int i;
  FILE *fp;
  struct zener *z1, *z2;
  struct zener_table *z;
  char line[160];
  
  if((fp=fopen(t->ad_file,"r")) == NULL){
      PVERB("Error opening file %s.\n\n",t->ad_file);
      exit(1);
  }
  z1 = z2 = (struct zener*) malloc(sizeof(struct zener));
  z1->next = NULL;
  while( fgets(line, sizeof(line), fp) ){
    if(sscanf(line,"%f %f",&z2->out,&z2->in) == 2){
      z2->out = z2->out - z2->in;
      z2->next = (struct zener*) malloc(sizeof(struct zener));
      z2 = z2->next;
      z2->next = NULL;
      count++;
    }
  }
  z = (struct zener_table*)malloc(sizeof(struct zener_table));
  z->size = count;
  z->in  =  (float*) malloc(sizeof(float)*count);
  z->out =  (float*) malloc(sizeof(float)*count);
  for(i=0;i<count;i++){
    z2=z1;
    *(z->in+i)  = z2->in;
    *(z->out+i) = z2->out;
    z1=z1->next;
    free(z2);
  }
  fclose(fp);
  return z;
}


int free_zener_table(struct instrument *t)
{
  free(t->zt->in);
  free(t->zt->out);
  free(t->zt);

  return 0;
}
  

float zener_cor(float v, struct instrument *t)
{
  float *x = t->zt->in;
  float *y = t->zt->out;
  int size = t->zt->size;
  int i=0;

  /* Extremos: */
  if(v > x[size-1]) return y[size-1];
  if(v < x[0]) return y[0];

  while( v >= *(x+i))i++;
  if( v == *(x+i) )return *(y+i);

  return ((v-x[i])*(y[i]-y[i-1])/(x[i]-x[i-1])+y[i]);
}
